<?php

namespace Drupal\preprocessors\PluginManager\PreprocessorsPluginManager;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a base class for Preprocessor plugins.
 *
 * @package Drupal\preprocessors\Plugin
 */
abstract class Preprocessor extends PluginBase implements PreprocessorInterface, ContainerFactoryPluginInterface {

  /**
   * Dependency injection create method override.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The Dependency Injection container.
   * @param array $configuration
   *   The plugin configuration.
   * @param string $plugin_id
   *   The plugin ID.
   * @param mixed $plugin_definition
   *   The plugin definition.
   *
   * @return static
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getId(): string {
    return $this->pluginDefinition['id'];
  }

  /**
   * {@inheritdoc}
   */
  public function getTemplate(): string {
    return $this->pluginDefinition['template'];
  }

  /**
   * {@inheritdoc}
   */
  abstract public function preprocess(&$variables, $hook, $info);

}
